import pkg_resources

from .simple_timing import SimpleTimer
try:
    from .progressbar_timing import IterTimer
except ImportError as exc:
    import sys
    if ((sys.version_info[0] < 3 and str(exc) == "No module named progressbar")
            or exc.name == 'progressbar'):
        def IterTimer(*args, **kwarg):  # pylint: disable=unused-argument
            raise ImportError("please install progressbar module using "
                              "\"pip install progressbar\" in python 2 or "
                              "\"pip install progressbar33\" in python 3")
    else:
        raise


__all__ = ['progressbar_timing', 'simple_timing']
__version__ = pkg_resources.get_distribution("bistiming").version



from functools import partial
import logging
import datetime

import six


class SimpleTimer(object):

    def __init__(self, description="", logger=None, logging_level=logging.INFO,
                 verbose_start=True, verbose_end=True, end_in_new_line=True,
                 prefix="..."):
        if logger is not None:
            self.log = partial(logger.log, logging_level)
        else:
            self.log = six.print_
        self.description = prefix + description
        self.verbose_start = verbose_start
        self.verbose_end = verbose_end
        self.end_in_new_line = end_in_new_line
        self.start_time = None
        self.end_time = None
        self.elapsed_time = None

    def __enter__(self):
        if self.verbose_start:
            if self.end_in_new_line:
                self.log(self.description)
            else:
                self.log(self.description, end="", flush=True)

        self.start_time = datetime.datetime.now()
        return self

    def __exit__(self, exc_type, exc, exc_tb):
        self.end_time = datetime.datetime.now()
        self.elapsed_time = self.end_time - self.start_time
        if self.verbose_end:
            if self.end_in_new_line:
                msg = self.description
            else:
                msg = ""
            msg += " done in {}".format(self.elapsed_time)
            self.log(msg)




import datetime
from progressbar import (ProgressBar, Percentage, UnknownLength,
                         Bar, SimpleProgress, Timer, ETA)


class PreciseTimer(Timer):
    """Widget which displays the elapsed seconds."""

    @staticmethod
    def format_time(seconds):
        """Formats time as the string "HH:MM:SS"."""
        return str(datetime.timedelta(seconds=seconds))


class PreciseETA(ETA):
    """Widget which attempts to estimate the time of arrival."""

    def update(self, pbar):
        """Updates the widget to show the ETA or total time when finished."""

        if pbar.currval == 0:
            return 'ETA:  --:--:--'
        elif pbar.finished:
            return 'Time: %s' % PreciseTimer.format_time(pbar.seconds_elapsed)
        else:
            elapsed = pbar.seconds_elapsed
            eta = elapsed * pbar.maxval / pbar.currval - elapsed
            return 'ETA:  %s' % PreciseTimer.format_time(eta)


class IterTimer(object):

    def __init__(self, name="", total=UnknownLength, period=1, verbose=3):
        """
        Args:
            verbose: 0: no output, do nothing
                     1: only output in start
                     2: only output in start and end
                     3: output in progresss
        """
        self.verbose = verbose
        self.period = period
        if verbose >= 2:
            widgets = ["...", name]
            if total is not UnknownLength:
                widgets.extend([" ", Bar(), " ", SimpleProgress("/"),
                                " (", Percentage(), ")  ", PreciseETA()])
            else:
                widgets.extend([" (", PreciseTimer(), ")"])
            self.pbar = ProgressBar(widgets=widgets, maxval=total)
        elif verbose >= 1:
            print("..." + name)

    def __enter__(self):
        if self.verbose >= 2:
            self.pbar.start()
            if self.verbose == 2:
                print("")
        return self

    def __exit__(self, exc_type, exc, exc_tb):
        if self.verbose >= 2:
            self.pbar.widgets.insert(2, " done")
            self.pbar.finish()

    def update(self, i, total=None):
        if i % self.period == 0:
            if self.verbose >= 2 and total is not None:
                self.pbar.maxval = total
            if self.verbose >= 3:
                self.pbar.update(i)



#!/usr/bin/env python
import os
from setuptools import setup


on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
# read the docs could not compile numpy and c extensions
if on_rtd:
    setup_requires = []
    install_requires = []
else:
    setup_requires = [
        'nose',
        'coverage',
    ]
    install_requires = [
        'six',
    ]

setup(
    name='bistiming',
    version="0.1.1",
    description='A timing and progress bar module for Python.',
    long_description=("See `github <https://github.com/ianlini/bistiming>`_ "
                      "for more information."),
    author='ianlini',
    url='https://github.com/ianlini/bistiming',
    setup_requires=setup_requires,
    install_requires=install_requires,
    classifiers=[
        'Topic :: Utilities',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ],
    test_suite='nose.collector',
    packages=[
        'bistiming',
    ],
    package_dir={
        'bistiming': 'bistiming',
    },
)
